
/*******************************************************************************
* Copyright (c) 2016 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
******************************************************************************/
#ifndef IDENTIFICATIONDATASOURCE_H
#define IDENTIFICATIONDATASOURCE_H


#include <pappsomspp/msrun/msrunid.h>
#include <pappsomspp/spectrum/spectrum.h>
#include <map>


class IdentificationDataSource
{
public:

    using MapIdentificationDataSources = std::map<QString, IdentificationDataSource *>;
    static IdentificationDataSource * getInstance(const QString & location);

    IdentificationDataSource(const QString resource_name);
    IdentificationDataSource(const IdentificationDataSource& other);
    ~IdentificationDataSource();
    bool operator==(const IdentificationDataSource& other) const;

    const QString & getResourceName () const;
    const QString getSampleName () const;
    void setMsRunSp (pappso::MsRunIdSp ms_run_sp);
    pappso::MsRunIdSp getMsRunSp () const;
    /** @brief get the spectrum with scan number
     * */
    virtual pappso::SpectrumSp getSpectrumSp(unsigned int scan_number) const;

protected :
    QString _resource_name;
private :
    static MapIdentificationDataSources _map_identification_data_sources;
    //static std::map<QString, pappso::MsRunIdSp> _map_msrunidsp;
    pappso::MsRunIdSp _ms_run_sp = nullptr;
};

#endif // IDENTIFICATIONDATASOURCE_H
