
/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "groupinggroup.h"
#include <core/proteinmatch.h>
#include <core/peptidematch.h>
#include <pappsomspp/grouping/grppeptide.h>
#include <pappsomspp/utils.h>
#include <set>

GroupingGroup::GroupingGroup()
{

}

GroupingGroup::~GroupingGroup()
{

}
unsigned int GroupingGroup::getNumberOfSubgroups() const {
    return _number_of_subgroup;
}

const QStringList GroupingGroup::getSubgroupIdList(const PeptideMatch * p_peptide_match) const {
       std::set<unsigned int> subgroup_list;
    pappso::GrpPeptide * p_grp_peptide = p_peptide_match->getGrpPeptideSp().get();
    if (p_grp_peptide != nullptr) {
    for (auto && pair_peptide_match :_pair_sg_number_peptide_match_list) {
        if(pair_peptide_match.second->getGrpPeptideSp().get() == p_grp_peptide) {
            subgroup_list.insert(pair_peptide_match.first);
        }
    }
    }
    
    
       QStringList sg_str_list;
       for (unsigned int sgnum : subgroup_list) {
           sg_str_list << QString("%1%2").arg(pappso::Utils::getLexicalOrderedString(_group_number)).arg(pappso::Utils::getLexicalOrderedString(sgnum));
       }
    return sg_str_list;

}

unsigned int GroupingGroup::countSubgroupPresence(const PeptideMatch * p_peptide_match) const {
    std::set<unsigned int> subgroup_list;
    pappso::GrpPeptide * p_grp_peptide = p_peptide_match->getGrpPeptideSp().get();
    if (p_grp_peptide != nullptr) {
    for (auto && pair_peptide_match :_pair_sg_number_peptide_match_list) {
        if(pair_peptide_match.second->getGrpPeptideSp().get() == p_grp_peptide) {
            subgroup_list.insert(pair_peptide_match.first);
        }
    }
    }
    return subgroup_list.size();
}

std::size_t GroupingGroup::countSpecificSpectrum(const ProteinMatch * p_protein_match) const {
    if (_number_of_subgroup == 1) {
        return p_protein_match->countValidAndCheckedSpectrum();
    }
    std::set<QString> spectrum_list_in;
    for (auto && p_peptide_match :p_protein_match->getPeptideMatchList()) {
        if (p_peptide_match->isValidAndChecked()) {
            spectrum_list_in.insert(QString("%1-%2").arg(p_peptide_match->getMsRunIdSp().get()->getXmlId()).arg(p_peptide_match->getScan()));
        }
    }
    std::set<QString> spectrum_list_out;
    unsigned int sg_number = p_protein_match->getGrpProteinSp().get()->getSubGroupNumber();
    for (auto && pair_peptide_match :_pair_sg_number_peptide_match_list) {
        if (pair_peptide_match.first != sg_number) {
            spectrum_list_out.insert(QString("%1-%2").arg(pair_peptide_match.second->getMsRunIdSp().get()->getXmlId()).arg(pair_peptide_match.second->getScan()));
        }
    }

    std::size_t count = 0;
    for (const QString & spectrum: spectrum_list_in) {
        std::set<QString>::const_iterator it = spectrum_list_out.find(spectrum);
        if (it == spectrum_list_out.end()) {
            count++;
        }
    }
    return count;
}

std::size_t GroupingGroup::countSpecificSequence(const ProteinMatch * p_protein_match) const {
    if (_number_of_subgroup == 1) {
        return p_protein_match->countUniqueSequence();
    }
    std::set<QString> sequence_list_in;
    for (auto && p_peptide_match :p_protein_match->getPeptideMatchList()) {
        if (p_peptide_match->isValidAndChecked()) {
            sequence_list_in.insert(p_peptide_match->getPeptideXtpSp().get()->getSequence());
        }
    }
    std::set<QString> sequence_list_out;
    unsigned int sg_number = p_protein_match->getGrpProteinSp().get()->getSubGroupNumber();
    for (auto && pair_peptide_match :_pair_sg_number_peptide_match_list) {
        if (pair_peptide_match.first != sg_number) {
            sequence_list_out.insert(pair_peptide_match.second->getPeptideXtpSp().get()->getSequence());
        }
    }

    std::size_t count = 0;
    for (const QString & sequence: sequence_list_in) {
        std::set<QString>::const_iterator it = sequence_list_out.find(sequence);
        if (it == sequence_list_out.end()) {
            count++;
        }
    }
    return count;
}

void GroupingGroup::add(const ProteinMatch * p_protein_match) {
_group_number = p_protein_match->getGrpProteinSp().get()->getGroupNumber();
    if (p_protein_match->getGrpProteinSp().get()->getRank() == 1) {
        _number_of_subgroup++;
        unsigned int sg_number = p_protein_match->getGrpProteinSp().get()->getSubGroupNumber();

        for (auto && p_peptide_match :p_protein_match->getPeptideMatchList()) {
            if (p_peptide_match->isValidAndChecked()) {
                _pair_sg_number_peptide_match_list.push_back(std::pair<unsigned int, const PeptideMatch * >(sg_number, p_peptide_match));
            }
        }
    }
}

