
/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#ifndef GROUPINGGROUP_H
#define GROUPINGGROUP_H

#include <memory>
#include <vector>
#include <QStringList>

class ProteinMatch;
class PeptideMatch;

class GroupingGroup;
typedef std::shared_ptr<GroupingGroup> GroupingGroupSp;

class GroupingGroup
{
public:

    GroupingGroup();
    ~GroupingGroup();

    void add(const ProteinMatch * p_protein_match);
    std::size_t countSpecificSpectrum(const ProteinMatch * p_protein_match) const;
    std::size_t countSpecificSequence(const ProteinMatch * p_protein_match) const;
    
    unsigned int getNumberOfSubgroups() const;
        
    /** @brief count number of subgroups in which this peptide is present
     * */
    unsigned int countSubgroupPresence(const PeptideMatch * p_peptide_match) const;
    
    /** @brief subgroup identifier list in which this peptide is present
     * */
    const QStringList getSubgroupIdList(const PeptideMatch * p_peptide_match) const;
    

private :
    unsigned int _group_number=0;
    unsigned int _number_of_subgroup=0;
    std::vector<std::pair<unsigned int, const PeptideMatch *>> _pair_sg_number_peptide_match_list;
};

#endif // GROUPINGGROUP_H
