
/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "peptidewindow.h"
#include "../project_view/projectwindow.h"
#include "ui_peptide_detail_view.h"
#include <pappsomspp/pappsoexception.h>
#include <pappsomspp/spectrum/qualifiedspectrum.h>
#include <QMessageBox>
#include <QSettings>

PeptideWindow::PeptideWindow(ProjectWindow *parent):
    QMainWindow(parent),
    ui(new Ui::PeptideDetailView)
{
    _p_project_window = parent;
    ui->setupUi(this);
    /*
    */

    QSettings settings;
    QString unit = settings.value("peptideview/precision_unit", "dalton").toString();
    pappso::mz precision = settings.value("peptideview/precision_value", "0.2").toDouble();

    if (unit == "dalton") {
        _p_precision = Precision::getDaltonInstance(precision);
    } else {
        _p_precision = Precision::getPpmInstance(precision);
    }

#if QT_VERSION >= 0x050000
    // Qt5 code
    /*
    connect(&workerThread, &QThread::finished, worker, &QObject::deleteLater);
    connect(this, &PtSpectrumViewer::operateMsDataFile, worker, &PwizLoaderThread::doMsDataFileLoad);
    connect(worker, &PwizLoaderThread::msDataReady, this, &PtSpectrumViewer::handleMsDataFile);
    */
#else
// Qt4 code
    connect (_p_project_window, SIGNAL(identificationGroupGrouped(IdentificationGroup *)), this,SLOT(doIdentificationGroupGrouped(IdentificationGroup *)));

    //connect(_protein_table_model_p, SIGNAL(layoutChanged()), this, SLOT(updateStatusBar()));
#endif
}

PeptideWindow::~PeptideWindow()
{
    delete ui;
}


void PeptideWindow::doIdentificationGroupGrouped(IdentificationGroup * p_identification_group) {
    updateDisplay();
}

void PeptideWindow::updateDisplay() {
    try {
        ui->sequence_label->setText(_p_peptide_match->getPeptideXtpSp().get()->toString());
        ui->mz_label->setText(QString("%1").arg(_p_peptide_match->getPeptideXtpSp().get()->getMz(_p_peptide_match->getCharge())));
        ui->z_label->setText(QString("%1").arg(_p_peptide_match->getCharge()));
        ui->scan_label->setText(QString("%1").arg(_p_peptide_match->getScan()));
        ui->modification_label->setText(_p_peptide_match->getPeptideXtpSp().get()->getModifString());
        
        
    }
    catch (pappso::PappsoException exception_pappso) {
        QMessageBox::warning(this,
                             tr("Unable to display protein details :"), exception_pappso.qwhat());
    }
    catch (std::exception exception_std) {
        QMessageBox::warning(this,
                             tr("Unable to display protein details :"), exception_std.what());
    }

}

void PeptideWindow::setPeptideMatch(PeptideMatch * p_peptide_match) {
    _p_peptide_match = p_peptide_match;
    
    SpectrumSp spectrum = _p_peptide_match->getIdentificationDataSource()->getSpectrumSp(_p_peptide_match->getScan());  
    pappso::QualifiedSpectrum spectrum_copy;
    spectrum_copy.setPrecursorCharge(_p_peptide_match->getCharge());
    spectrum_copy.setOriginalSpectrumSp(spectrum);

    pappso::PeptideSp peptide = p_peptide_match->getPeptideXtpSp();
    ui->spectrumWidget->setPrecision(_p_precision);
    ui->spectrumWidget->setPeptideSp(peptide);
    ui->spectrumWidget->setQualifiedSpectrum(spectrum_copy);
    updateDisplay();
}
